/*
	This file is part of Posixcpp library.

	Posixcpp library is free software: you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public License
	as published by the Free Software Foundation, either version 3 of
	the License, or (at your option) any later version.

	Posixcpp library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with Posixcpp library. If not, see
	<http://www.gnu.org/licenses/>.
 */

/**
 * @file posix_io_file.hpp
 */

#ifndef POISX_IO_FILE_HPP
#define POISX_IO_FILE_HPP

#include "posix_io_handle.hpp"

#include <string>

POSIX_BEGIN_NAMESPACE

namespace io
{

class IFile : public virtual IHandle
{
public:
	virtual ~IFile() noexcept = default;

	/**
	 * @brief  Create file
	 */
	virtual void creat() = 0;

	/**
	 * @brief Overloaded version
	 * @param fileName file name to cretae
	 * @param mode
	 */
	virtual void creat(
	    const std::string &fileName,
	    Mode mode = Mode_t::NONE,
	    CreateFlags cMode = CreateMode_t::NONE) = 0;

	/**
	 * @brief Open file
	 */
	virtual void open() = 0;

	/**
	 * @brief Overloaded version
	 * @param fileName
	 * @param mode
	 */
	virtual void open(
	    const std::string &fileName,
	    Mode mode = Mode_t::NONE,
	    CreateFlags cFlags = CreateMode_t::NONE) = 0;

	/**
	 * @brief Delete file
	 */
	virtual void unlink() = 0;

	/**
	 * @brief Overloaded version
	 * @param fileName
	 */
	virtual void unlink(const std::string &fileName) = 0;

	virtual void setFileName(const std::string &str) = 0;
};

class File : public Handle, public IFile
{
public:

	File();

	File(int);

	File(const std::string&);

	/**
	 * Create file
	 */
	void creat() override;

	void creat(
	    const std::string& fileName,
	    Mode mode = Mode_t::NONE,
	    CreateFlags cMode = CreateMode_t::NONE) override;

	void open() override;

	void open(
	    const std::string& fileName,
	    Mode mode = Mode_t::NONE,
	    CreateFlags cFlags = CreateMode_t::NONE) override;

	void unlink() override;

	void unlink(const std::string& fileName) override;

	File dup();

	void dup(File &old);

	void setFileName(const std::string &str) override
	{
		_fileName = str;
	}

	const std::string &fileName() const;

	void setAccessMode(AccessMode_t newVal)
	{
		_accessMode = newVal;
	}

	AccessMode_t accessMode() const
	{
		return (_accessMode);
	}

	void setMode(Mode newVal)
	{
		_mode = newVal;
	}

	Mode mode() const
	{
		return (_mode);
	}

	int customFlags() const
	{
		return (_customFlags);
	}

	void setCustomFlags(int newVal)
	{
		_customFlags = newVal;
	}
	
	CreateFlags	createFlags() const
	{
		return (_cMode);
	}
	void setCretaeFlags(CreateFlags newVal)
	{
		_cMode = newVal;
	}

private:
	std::string	_fileName;
	AccessMode_t	_accessMode;
	CreateFlags	_cMode;
	int		_customFlags;
	Mode		_mode;
};

}

POSIX_END_NAMESPACE

#endif /* POISX_IO_FILE_HPP */
