/*
    This file is part of Posixcpp library.

    Posixcpp library is free software: you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public License
    as published by the Free Software Foundation, either version 3 of
    the License, or (at your option) any later version.

    Posixcpp library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with Posixcpp library. If not, see
    <http://www.gnu.org/licenses/>.
 */

#ifndef POSIX_SOCKETS_HPP
#define POSIX_SOCKETS_HPP

#include "posix_io.hpp"

#include <sys/socket.h>

namespace posix
{

namespace io
{

namespace sockets
{

/**
 * Socket types
 */
enum class Type_t
{
	UNDEFINED = 0,
	/**
	 * Byte-stream socket.
	 */
	STREAM = SOCK_STREAM,
	/**
	 * Datagram socket.
	 */
	DATAGRAM = SOCK_DGRAM,
	/**
	 * Raw protocol interface.
	 */
	RAW = SOCK_RAW,
	/**
	 * Sequenced-packet socket.
	 */
	SEQPACKET = SOCK_SEQPACKET
};

enum class Family_t : sa_family_t
{
	/**
	 * Local
	 */
	LOCAL = PF_LOCAL,
	/**
	 * IP
	 */
	INET = PF_INET,
	/**
	 * IPv6
	 */
	INET6 = PF_INET6,
	/**
	 * Erthernet?
	 */
	PACKET = PF_PACKET,
	/**
	 * LLC
	 */
	LLC = PF_LLC,
	UNSPECIFIED = PF_UNSPEC
};

enum class Flags_t
{
	NONE = 0,
	OUT_OF_BOUND = MSG_OOB,
	END_OF_RECORD = MSG_EOR,
	DONTWAIT = MSG_DONTWAIT,
	MORE = MSG_MORE
};

enum class ShutdownType_t
{
	READ = SHUT_RD,
	WRITE = SHUT_WR,
	READ_WRITE = SHUT_RDWR
};

class Address;

class PhisicalAddress;

class InetAddress;

class Socket;

}

}

}

#endif /* POSIX_SOCKETS_HPP */
