/*
    This file is part of Posixcpp library.

    Posixcpp library is free software: you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public License
    as published by the Free Software Foundation, either version 3 of
    the License, or (at your option) any later version.

    Posixcpp library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with Posixcpp library. If not, see
    <http://www.gnu.org/licenses/>.
 */

#ifndef POSIX_THREADS_BARRIER_HPP_INCLUDED
#define POSIX_THREADS_BARRIER_HPP_INCLUDED

#include "posix_threads.hpp"
#include "posixcpp_handlewrapper.hpp"

namespace posix
{
namespace threads
{

/**
 * @brief Threads barrier
 */
class Barrier final
{
public:

	class Attributes : public posixcpp::HandleWrapper<pthread_barrierattr_t>
	{
	public:
		Attributes();
		
		void setPShared(ProcessShared_t);
		ProcessShared_t getPSHared();
	};

	Barrier(unsigned int count);

	~Barrier();

	/**
	 * wait for other threads
	 */
	bool wait();

	pthread_barrier_t & handle()
	{
		return _handle;
	}

	const pthread_barrier_t & handle() const
	{
		return _handle;
	}

	int count() const
	{
		return m_count;
	}

private:

	pthread_barrier_t _handle;

	int m_count;
};

}
}

#endif // POSIX_THREADS_BARRIER_HPP_INCLUDED
