/*
    This file is part of Posixcpp library.

    Posixcpp library is free software: you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public License
    as published by the Free Software Foundation, either version 3 of
    the License, or (at your option) any later version.

    Posixcpp library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with Posixcpp library. If not, see
    <http://www.gnu.org/licenses/>.
 */

/**
 * @file criticalsection.hpp
 * @brief Locker
 */

#ifndef POSIX_THREADS_CRITICALSECTION_INCLUDED
#define POSIX_THREADS_CRITICALSECTION_INCLUDED

#include "posix_threads_ilock.hpp"
#include "posixcpp.hpp"

#define CS_DEBUG 0

#if CS_DEBUG
#include "posix_threads_mutex.hpp"
#endif

namespace posix
{

namespace threads
{

/**
 * @brief Critical section
 *
 * The instance marks section of code as critical section during its existance
 */
class CriticalSection final
{
public:
    CriticalSection(posix::threads::ILock & lock);
    
    ~CriticalSection();
private:
    posix::threads::ILock & m_lock;
#if CS_DEBUG
    unsigned m_id;
    static unsigned m_idCounter;
    static posix::threads::Mutex m_csLock;
#endif
};

}

}

#endif // POSIX_THREADS_CRITICALSECTION_INCLUDED
