/*
    This file is part of Posixcpp library.

    Posixcpp library is free software: you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public License
    as published by the Free Software Foundation, either version 3 of
    the License, or (at your option) any later version.

    Posixcpp library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with Posixcpp library. If not, see
    <http://www.gnu.org/licenses/>.
*/

/**
 * @file ilock.hpp
 * @brief binary semaphore interface header
 */

#ifndef POSIX_THREADS_ILOCK_INCLUDED
#define POSIX_THREADS_ILOCK_INCLUDED

#include "posixcpp.hpp"


namespace posix
{
namespace threads
{
/**
 * @brief Lock interface
 */
class ILock
{
	POSIXCPP_INTERFACE(ILock)
public:

	/**
	 * @brief lock resource
	 */
	virtual void lock() = 0;
	/**
	 * @brief unlock resource
	 */
	virtual void unlock() = 0;
	/**
	 * @brief try lock the resource
	 * @return true if locked, false on failure
	 */
	virtual bool tryLock() = 0;
};

}
}
#endif // POSIX_THREADS_ILOCK_INCLUDED
