/*
	This file is part of Posixcpp library.

	Posixcpp library is free software: you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public License
	as published by the Free Software Foundation, either version 3 of
	the License, or (at your option) any later version.

	Posixcpp library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with Posixcpp library. If not, see
	<http://www.gnu.org/licenses/>.
 */

#ifndef POSIX_TIME_CLOCK_HPP
#define	POSIX_TIME_CLOCK_HPP

#include <map>

#include "posix_time.hpp"
#include "posix_time_timespec.hpp"

namespace posix
{
namespace time
{

class Clock final
{
    
public:
	enum class SleepMode_t
	{
		RELATIVE = 0,
		ABSOLUTE = TIMER_ABSTIME
	};

	Clock(Clock_t clockId);
	
	~Clock();

	friend Clock & clock(Clock_t id);

	/**
	 * @property Res
	 * @brief Clock resolution
	 */
    
	/**
	 * @brief Get Res property
	 */
	Timespec getRes() const;

	/**
	 * @property Time
	 * @brief Value of the clock
	 */

	/**
	 * @brief Time property get
	 */
	Timespec getTime() const;
	/**
	 * @brief Time property set
	 */
	void setTime(const Timespec &time);

	Timespec nanoSleep(
	  const Timespec &required,
	  SleepMode_t mode = SleepMode_t::RELATIVE) const;
    
	Clock_t clockId() const
	{
		return m_clockId;
	}

private:
	
	class StaticInitializer
	{
	private:
		friend class Clock;
		StaticInitializer();
		~StaticInitializer();
	};
	
	friend class StaticInitializer;

	Clock()
	{
	}

	static std::map<Clock_t, Clock> m_clocks;
	
	static StaticInitializer m_initializer;

	Clock_t m_clockId;
};

Clock & clock(Clock_t id);

}
}

#endif	/* POSIX_TIME_CLOCK_HPP */

