/*
    This file is part of Posixcpp library.

    Posixcpp library is free software: you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public License
    as published by the Free Software Foundation, either version 3 of
    the License, or (at your option) any later version.

    Posixcpp library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with Posixcpp library. If not, see
    <http://www.gnu.org/licenses/>.
 */

#ifndef POSIXCPP_HANDLEWRAPPER_HPP
#define POSIXCPP_HANDLEWRAPPER_HPP

#include "posixcpp.hpp"


namespace posixcpp
{

template <class Native>
class HandleWrapper
{
public:

	virtual ~HandleWrapper()
	{
		POSIX_CALL_ASSERT((errno = m_destructor(&m_handle)) == 0);
	}

	const Native &handle() const
	{
		return m_handle;
	}

	Native &handle()
	{
		return m_handle;
	}

protected:

	HandleWrapper(int (*constructor)(Native *),
	    int (*destructor)(Native *)) :
	    m_constructor(constructor),
	    m_destructor(destructor)
	{
		POSIX_CALL_ASSERT((errno = m_constructor(&m_handle)) == 0);
	}

	Native m_handle;

private:
	int (*m_constructor)(Native*);
	int (*m_destructor)(Native*);
};

}

#endif
