/*
    This file is part of Posixcpp library.

    Posixcpp library is free software: you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public License
    as published by the Free Software Foundation, either version 3 of
    the License, or (at your option) any later version.

    Posixcpp library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with Posixcpp library. If not, see
    <http://www.gnu.org/licenses/>.
*/

#include "posix_io.hpp"

using namespace posix;
using namespace posix::io;

POSIX_BEGIN_NAMESPACE

namespace io
{

std::ostream&
operator <<(
   std::ostream &stream,
#if __cplusplus >= 201103
  ControlCharacters_t cChar)
#else
  io::ControlCharacters_t::Enum	cChar)
#endif
{
	switch (cChar) {
	case io::ControlCharacters_t::NUL : stream << "NUL (0x00)"; break;
	case io::ControlCharacters_t::SOH : stream << "SOH (0x01)"; break;
	case io::ControlCharacters_t::STX : stream << "STX (0x02)"; break;
	case io::ControlCharacters_t::ETX : stream << "ETX (0x03)"; break;
	case io::ControlCharacters_t::EOT : stream << "EOT (0x04)"; break;
	case io::ControlCharacters_t::ENQ : stream << "ENQ (0x05)"; break;
	case io::ControlCharacters_t::ACK : stream << "ACK (0x06)"; break;
	case io::ControlCharacters_t::BEL : stream << "BEL (0x07)"; break;
	case io::ControlCharacters_t::BS : stream << "BS (0x08)"; break;
	case io::ControlCharacters_t::HT : stream << "HT (0x09)"; break;
	case io::ControlCharacters_t::LF : stream << "LF (0x0A)"; break;
	case io::ControlCharacters_t::VT : stream << "VT (0x0B)"; break;
	case io::ControlCharacters_t::FF : stream << "FF (0x0C)"; break;
	case io::ControlCharacters_t::CR : stream << "CR (0x0D)"; break;
	case io::ControlCharacters_t::NAK : stream << "NAK (0x15)"; break;
	case io::ControlCharacters_t::CAN : stream << "CAN (0x18)"; break;
	}
	
	return (stream);
}

}

POSIX_END_NAMESPACE
