/*
    This file is part of Posixcpp library.

    Posixcpp library is free software: you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public License
    as published by the Free Software Foundation, either version 3 of
    the License, or (at your option) any later version.

    Posixcpp library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with Posixcpp library. If not, see
    <http://www.gnu.org/licenses/>.
 */

#include "posix_io_file.hpp"
#include "posix_io_handle.hpp"

#include <limits.h>
#include <fcntl.h>

using namespace posix::io;
using namespace posix;

File::File() :
_accessMode(AccessMode_t::READ_ONLY),
_customFlags(0),
_mode(Mode_t::RWXU)
{
}

File::File(int descr) :
Handle(descr),
_accessMode(AccessMode_t::READ_ONLY),
_customFlags(0),
_mode(Mode_t::RWXU)
{
}

File::File(const std::string &fileName) :
_fileName(fileName),
_accessMode(AccessMode_t::READ_ONLY),
_customFlags(0),
_mode(Mode_t::RWXU)
{
}

const std::string&
File::fileName() const
{
	//if (descriptor() == -1)
		return (_fileName);
	/*else {
		char filePath[PATH_MAX];
		POSIX_CALL_ASSERT(::fcntl(descriptor(), F_GETPATH, filePath) != -1);
	 return (_fileName = filePath);
	}*/
}

void
File::creat()
{
	int desc;
	
	PRECONDITION_ASSERT(descriptor() == -1, "file already opened");
	PRECONDITION_ASSERT(!_fileName.empty(), "empty file name");
	PRECONDITION_ASSERT(_fileName != "", "empty file name");

	POSIX_CALL_ASSERT((desc = ::creat(_fileName.c_str(),
	    _mode.data())) != -1);
	
	setDescriptor(desc);
}

void
File::creat(
    const std::string &fileName,
    Mode mode,
    CreateFlags cMode)
{
	this->setFileName(fileName);
	this->setMode(mode);
	
	this->creat();
}

void
io::File::open()
{
	int desc;
	
	PRECONDITION_ASSERT(descriptor() == -1, "file already opened");
	PRECONDITION_ASSERT(!_fileName.empty(), "empty file name");
	PRECONDITION_ASSERT(_fileName != "", "empty file name");

	std::string message("Unable to open file (");
	message.append(_fileName).append(")");

	if (_mode.data() == 0) {
		POSIX_CALL_ASSERT_X((desc = ::open(_fileName.c_str(),
		    int(_accessMode) | int(blockingMode()) | _cMode.data() |
		    _customFlags)) != -1, message.c_str());
	} else {
		POSIX_CALL_ASSERT_X((desc = ::open(_fileName.c_str(),
		    int(_accessMode) | int(blockingMode()) | _cMode.data() |
		    _customFlags, _mode.data())) != -1, message.c_str());
	}
	
	setDescriptor(desc);
}

void
io::File::open(
    const std::string& fileName,
    Mode mode,
    CreateFlags cFlags)
{
	this->setFileName(fileName);
	this->setMode(mode);
	this->setCretaeFlags(cFlags);
	this->open();
}

void
io::File::unlink()
{
	PRECONDITION_ASSERT(descriptor() == -1, "file was opened");
	PRECONDITION_ASSERT(!_fileName.empty(), "empty file name");
	PRECONDITION_ASSERT(_fileName != "", "empty file name");

	POSIX_CALL_ASSERT_X(::unlink(_fileName.c_str()) == 0,
	    std::string("Unable to unlink file ").append(_fileName));
}

void
io::File::unlink(const std::string& fileName)
{
	this->setFileName(fileName);
	this->unlink();
}

io::File
io::File::dup()
{
	File result;
	int desc;
	
	POSIX_CALL_ASSERT((desc = ::dup(this->descriptor())) != -1);
	result.setDescriptor(desc);
	result.setFileName(this->fileName());
	
	return (result);
}

void
io::File::dup(File& newFile)
{
	int desc;

	POSIX_CALL_ASSERT((desc = ::dup2(this->descriptor(),
	    newFile.descriptor())) != -1);

	newFile.setDescriptor(desc);
	newFile.setFileName(this->fileName());
}
