/*
    This file is part of Posixcpp library.

    Posixcpp library is free software: you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public License
    as published by the Free Software Foundation, either version 3 of
    the License, or (at your option) any later version.

    Posixcpp library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with Posixcpp library. If not, see
    <http://www.gnu.org/licenses/>.
 */

#include "posix_io_i2c.hpp"

#include <sys/ioctl.h>
#include <linux/i2c-dev.h>

using namespace posix;

io::I2C::I2C(const std::string &path) :
File(path)
{
}

void io::I2C::setTimeout(posix::time::Timespec newVal)
{
	unsigned long	tout;
	
	tout = newVal.sec() * 100 + newVal.nsec() / 10000000ul;
	POSIX_CALL_ASSERT(::ioctl(descriptor(), I2C_TIMEOUT, tout) == 0);
}

void io::I2C::setRetries(unsigned long newVal)
{
	POSIX_CALL_ASSERT(::ioctl(descriptor(), I2C_RETRIES, newVal) == 0);
}

void io::I2C::setSlave(unsigned long address, bool force)
{
	unsigned long req;
	
	if (force)
		req = I2C_SLAVE_FORCE;
	else
		req = I2C_SLAVE;
	POSIX_CALL_ASSERT(::ioctl(descriptor(), req, address) == 0);
}
