/*
    This file is part of Posixcpp library.

    Posixcpp library is free software: you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public License
    as published by the Free Software Foundation, either version 3 of
    the License, or (at your option) any later version.

    Posixcpp library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with Posixcpp library. If not, see
    <http://www.gnu.org/licenses/>.
 */

#include <cstdlib>

#include "posixcpp.hpp"
#include "posix_io_pseudotty.hpp"

using namespace posix;

io::PseudoTTY::PseudoTTY()
{
}

void
io::PseudoTTY::get()
{
	int desc;
	
	POSIX_CALL_ASSERT((desc = ::getpt()) != -1);
	
	setDescriptor(desc);
}

void
io::PseudoTTY::open()
{
	this->get();
}

void
io::PseudoTTY::grant() const
{
	PRECONDITION_ASSERT(descriptor() > -1, "");
	
	POSIX_CALL_ASSERT(::grantpt(descriptor()) == 0);
}

void 
io::PseudoTTY::unlock() const
{
	PRECONDITION_ASSERT(descriptor() > -1, "");
	
	POSIX_CALL_ASSERT(::unlockpt(descriptor()) == 0);
}

std::string
io::PseudoTTY::slaveName() const
{
	char *result;
	POSIX_CALL_ASSERT((result=::ptsname(descriptor()))!=NULL);
	return (result);
}
