/*
    This file is part of Posixcpp library.

    Posixcpp library is free software: you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public License
    as published by the Free Software Foundation, either version 3 of
    the License, or (at your option) any later version.

    Posixcpp library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with Posixcpp library. If not, see
    <http://www.gnu.org/licenses/>.
*/

#include "posix_threads_barrier.hpp"

using namespace posix::threads;

Barrier::Attributes::Attributes() :
posixcpp::HandleWrapper<pthread_barrierattr_t>(&pthread_barrierattr_init,
    &pthread_barrierattr_destroy)
{
}

void
Barrier::Attributes::setPShared(ProcessShared_t newVal)
{
	POSIX_CALL_ASSERT((errno=pthread_barrierattr_setpshared(&m_handle,
	    int(newVal))) == 0);
}

ProcessShared_t
Barrier::Attributes::getPSHared()
{
	int result;
	
	POSIX_CALL_ASSERT((errno=pthread_barrierattr_getpshared(&m_handle,
	    &result)) == 0);
	
	return (ProcessShared_t(result));
}

Barrier::Barrier(unsigned int count) :
m_count(count)
{
    POSIX_CALL_ASSERT_X((errno=pthread_barrier_init(&this->_handle,
        NULL, m_count))==0, "Unable to initialize barrier");
}

Barrier::~Barrier()
{
    POSIX_CALL_ASSERT_X((errno=pthread_barrier_destroy(&this->_handle))==0,
        "Unable to desctroy barrier");
}

bool
Barrier::wait()
{
    POSIX_CALL_ASSERT_X( (errno=pthread_barrier_wait(&this->_handle))==0,
        "Unable to desctroy barrier" );
    return true;
}
