/*
    This file is part of Posixcpp library.

    Posixcpp library is free software: you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public License
    as published by the Free Software Foundation, either version 3 of
    the License, or (at your option) any later version.

    Posixcpp library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with Posixcpp library. If not, see
    <http://www.gnu.org/licenses/>.
*/

/**
 * @file criticalsection.cpp
 * @brief CS class implementation
 */

#include "posix_threads_criticalsection.hpp"
#include "posixcpp.hpp"
#include "posix_threads_mutex.hpp"

using posix::threads::CriticalSection;
using namespace posix::threads;

#if CS_DEBUG
#include <iostream>
unsigned CriticalSection::m_idCounter = unsigned(-1);
posix::threads::Mutex CriticalSection::m_csLock;
#endif

CriticalSection::CriticalSection(posix::threads::ILock & lock) :
m_lock(lock)
{
	m_lock.lock();
#if CS_DEBUG
	m_csLock.lock();
	m_id = ++m_idCounter;
	std::cout << __PRETTY_FUNCTION__ << ':' << m_id << std::endl;
	m_csLock.unlock();
#endif
}

CriticalSection::~CriticalSection()
{
	m_lock.unlock();
	
#if CS_DEBUG
	m_csLock.lock();
	std::cout << __PRETTY_FUNCTION__ << ':' << m_id << std::endl;
	m_csLock.unlock();
#endif
}
