/*
    This file is part of Posixcpp library.

    Posixcpp library is free software: you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public License
    as published by the Free Software Foundation, either version 3 of
    the License, or (at your option) any later version.

    Posixcpp library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with Posixcpp library. If not, see
    <http://www.gnu.org/licenses/>.
*/

#include "posix_threads_rwlock.hpp"

using namespace posix::threads;

RWLock::Attributes::Attributes() :
posixcpp::HandleWrapper<pthread_rwlockattr_t>(&pthread_rwlockattr_init,
    &pthread_rwlockattr_destroy)
{
}

void
RWLock::Attributes::setPShared(ProcessShared_t newVal)
{
	POSIX_CALL_ASSERT((errno=pthread_rwlockattr_setpshared(&m_handle,
	    int(newVal))) == 0);
}

ProcessShared_t
RWLock::Attributes::getPSHared()
{
	int result;
	
	POSIX_CALL_ASSERT((errno=pthread_rwlockattr_getpshared(&m_handle,
	    &result)) == 0);
	
	return (ProcessShared_t(result));
}

RWLock::RWLock(Attributes *attrs)
{
	POSIX_CALL_ASSERT((errno=pthread_rwlock_init(&_handle, 
	    attrs ? &attrs->handle() : NULL)) == 0);
}

RWLock::~RWLock()
{
	POSIX_CALL_ASSERT((errno=pthread_rwlock_destroy(&_handle)) == 0);
}

void RWLock::rdlock()
{
	POSIX_CALL_ASSERT((errno=pthread_rwlock_rdlock(&_handle)) == 0);
}

void RWLock::wrlock()
{
	POSIX_CALL_ASSERT((errno=pthread_rwlock_wrlock(&_handle)) == 0);
}

void RWLock::unlock()
{
	POSIX_CALL_ASSERT((errno=pthread_rwlock_unlock(&_handle)) == 0);
}
